# ALMA Data Reduction Script - imaging and self-calibration
# Using DSHARP HD 163296 data from project 2016.1.00484.L

# HD163296_cont_all.ms is the target continuum data with instrumental,  bandpass,
# flux scale and phase-ref corrections applied.
# HD163296_CO_new.ms is the target CO line data with instrumental,  bandpass,
# flux scale and phase-ref corrections applied.
# These were produced from running the provided scriptforPI.py associated with
# this source and project on the raw data downloaded from the ALMA archive.

# The ms file resulting from this (calibrated_final.ms) was averaged and split
# to produce HD163296_cont_all.ms and HD163296_CO_new.ms (see step 0 below).
# This was done for the purpose of the tutorial to reduce the computing power
# and time required to image the data.
# IMPORTANT NOTE: The continuum data now contains all continuum spws, so the
# relevant commands have been changed and certain tasks (e.g. tclean) take a
# little longer. The line data now only contains one combined spw.

# Set mysteps on commandline or uncomment below
# mysteps = [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23]

thesteps = []
step_title = {#0: 'Average data',
              1: 'Listobs, save flags and plotants',
              2: 'First continuum image',
              3: 'First self-calibration continuum image',
              4: 'Phase self-calibration p1',
              5: 'Re-image continuum applying p1',
              6: 'Phase self-calibration p2',
              7: 'Re-image continuum applying p1 and p2',
              8: 'Phase self-calibration p3',
              9: 'Re-image continuum applying p1, p2 and p3',
              10: 'Amp self-calibration a1 (applying p1-3)',
              11: 'Re-image continuum applying p1-3 and a1',
              12: 'Apply p1-3 and a1 to line data, plot uv spectra',
              13: 'Subtract continuum from line',
              14: 'Make a CO(2-1) spectral cube with manual masking',
              15: 'Make a CO(2-1) spectral cube with automasking',
              16: 'Make spectrum, write out to file and fit spectral line',
              17: 'Make zeroth moment map',
              18: 'Make first and second moment maps',
              19: 'Correct for the primary beam response',
              20: 'Fit the continuum emission with a 2D gaussian',
              21: 'Make pv diagram using the viewer',
              22: 'Make pv diagram using task impv',
              23: 'Reproject an image to Galatic coordinates'}

try:
  print('List of steps to be executed ...', mysteps)
  thesteps = mysteps
except:
  print('global variable mysteps not set.')

if (thesteps==[]):
  thesteps = range(0,len(step_title))
  print('Executing all steps: ', thesteps)

interact=True    # Set to False to run unattendended after setting variables
# If True, close the plotter window every time you have finished

# The Python variable 'mysteps' will control which steps
# are executed when you start the script using
#   execfile('scriptForCalibration.py')
# e.g. setting mysteps = [2,3,4]# before starting the script will
# make the script execute only steps 2, 3, and 4
# Setting mysteps = [] will make it execute all steps.
# This can be done on the commandline or by uncommenting and
# editing the line # mysteps = [...] (line 16 above)

# Average and concatenate data (already done)
#mystep = 0
#if(mystep in thesteps):
#    casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
#    print 'Step ', mystep, step_title[mystep]
#
# Save data flags before flagging CO line
#flagmanager(vis='calibrated_final.ms', mode='save',
#            versionname='before_line_flags')
#
# Flag CO line channels before averaging continuum
#flagdata(vis='calibrated_final.ms', spw='3:1853~1965,7:1853~1965')
#
# Average data without CO line
#os.system('rm -rf HD163296_cont_spw37.ms')
#mstransform(vis='calibrated_final.ms', outputvis='HD163296_cont_spw37.ms',
#            spw='3,7', timeaverage=True,  timebin='120s',
#            datacolumn='data',regridms=True, mode='channel', width=128,
#            combinespws=True)
#
# Restore data flags to unflag CO line
#flagmanager(vis='calibrated_final.ms', mode='restore',
#            versionname='before_line_flags')
#
# Split out TDM continuum spws
#os.system('rm -rf HD163296_cont_spw012.ms')
#mstransform(vis='calibrated_final.ms', outputvis='HD163296_cont_spw012.ms',
#            spw='0,1,2', timeaverage=True,  timebin='120s',
#            datacolumn='data',regridms=True)
#
#os.system('rm -rf HD163296_cont_spw456.ms')
#mstransform(vis='calibrated_final.ms', outputvis='HD163296_cont_spw456.ms',
#            spw='4,5,6', timeaverage=True,  timebin='120s',
#            datacolumn='data',regridms=True)
#
# Concatenate continuum spectral windows
#os.system('rm -rf HD163296_cont_all.ms')
#concat(vis=['HD163296_cont_spw012.ms',''HD163296_cont_spw456.ms','HD163296_cont_spw37.ms'],
#       concatvis='HD163296_cont_all.ms', freqtol='1MHz')
#
#
#os.system('rm -rf HD163296_CO_new.ms')
#mstransform(vis='calibrated_final.ms', outputvis='HD163296_CO_new.ms',
#            spw='3,7', timeaverage=True,  timebin='120s',
#            datacolumn='data',regridms=True, mode='channel',
#            start=1863, width=3, nchan=37, outframe='LSRK',
#            veltype='radio', restfreq='230.538GHz', combinespws=True)

# Listobs, save flags and plotants
mystep = 1
if(mystep in thesteps):
    casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
    print('Step ', mystep, step_title[mystep])

    os.system('rm -rf HD163296_cont_all.ms.listobs')
    listobs(vis = 'HD163296_cont_all.ms',
          listfile = 'HD163296_cont_all.ms.listobs')

    if not os.path.exists('HD163296_cont_all.ms.flagversions/flags.Original'):
      flagmanager(vis = 'HD163296_cont_all.ms',
                  mode = 'save',
                  versionname = 'Original')

    os.system('rm -rf HD163296_CO_new.ms.listobs')
    listobs(vis = 'HD163296_CO_new.ms',
          listfile = 'HD163296_CO_new.ms.listobs')

    if not os.path.exists('HD163296_CO_new.ms.flagversions/flags.Original'):
      flagmanager(vis = 'HD163296_CO_new.ms',
                  mode = 'save',
                  versionname = 'Original')

    if interact==True:
      plotants(vis = 'HD163296_cont_all.ms', logpos=True,
             figfile='HD163296_cont_all.plotants.png')

      print('When you are done with and have closed the graphics window,')
      user_check = input('press enter to continue script\n')

# antref = reference antenna(s) (with good data, near array centre)
# use plotants or weblog to select this
antrefs='***'

# From plotants roughly estimate the longest baseline and thus the
# pixel size to get at least 5 pixels across the synthesized beam.
# The listobs file will give you the central frequency.
cellsize='***arcsec'

# Using the sensitivity calculator for the observing conditions,
# and the current bandwidth predicts a continuum rms ~0.02 mJy.

# First pre-selfcal continuum image
mystep = 2
if(mystep in thesteps):
    casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
    print('Step ', mystep, step_title[mystep])

    if antrefs !='***' and cellsize != '***arcsec':
        
        os.system('rm -rf HD163296_cont.firstclean*')
        tclean(vis='HD163296_cont_all.ms',
               imagename='HD163296_cont.firstclean',
               specmode='mfs', deconvolver='multiscale',
               imsize=500, spw='0~3', scales=[0, 5, 10],
               cell=cellsize, weighting='briggs',
               robust=-0.5, interactive=interact,
               threshold='0.2mJy', niter=10000) # Unlikely will be able to clean deeply
    else:
        print('You did not set the antrefs and cellsize parameters.')
        print('Please set these parameters in the script before running step 2 again.')

# First selfcal continuum image
mystep = 3
if(mystep in thesteps):
    casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
    print('Step ', mystep, step_title[mystep])

    # delete existing models and calibration tables to make sure start from clean dataset
    delmod(vis='HD163296_cont_all.ms', scr=True)
    clearcal(vis='HD163296_cont_all.ms')

    os.system('rm -rf HD163296_cont.clean*')
    tclean(vis='HD163296_cont_all.ms',
           imagename='HD163296_cont.clean',
           specmode='mfs', deconvolver='multiscale',
           imsize=500, spw='0~3', scales=[0, 5, 10],
           cell=cellsize, weighting='briggs',
           robust=-0.5, interactive=interact,
           threshold='0.1mJy', niter=300)

    peak=imstat(imagename='HD163296_cont.clean.image',box='125,125,375,375')['max'][0]
    rms=imstat(imagename='HD163296_cont.clean.image',box='10,10,125,125')['rms'][0]
    print('HD163296_cont.clean.image peak = %7.4f mJy/beam, rms  = %7.4f mJy/beam, S/N = %7.4f' % (peak*1000, rms*1000, peak/rms))
    print('Make a note of these numbers to check for improvement after self-cal.')

    # *@* Calculation of min solution interval
    ToS = 54.33*60. # seconds
    rms = 0.02      # ~theoretical rms for the continuum for these observing params across bandwidth, mJy/beam
    P = 3.0         # continuum peak flux density on long baselines, mJy/beam (start with continuum peak)
    N = 40.         # number of good antennas
    npol = 2.       # number of polarisations or correlations
    nspw = 4.       # effective number of continnum spws
    minsol = (3.*rms/P)**2 * ToS * (N*(N-1)/(2.*(N-3))) * npol * nspw
    print('Minimum solint for S/N 3 per antenna, per spw, per polarization %5.1f seconds' % (minsol))

# Phase self-cal
mystep = 4
if mystep in thesteps:
    casalog.post('Step '+str(mystep)+' '+step_title[mystep], 'INFO')
    print('Step ', mystep, step_title[mystep])

    # Insert model from image in ms
    ft(vis='HD163296_cont_all.ms',
       model='HD163296_cont.clean.model',
       usescratch=True)

    # check that model has saved after ft
    if interact:
       os.system('rm HD163296_cont_all.clean.model.png')
       plotms(vis='HD163296_cont_all.ms', xaxis='uvwave', yaxis='amp', ydatacolumn='model',
              showgui=interact, plotfile='HD163296_cont_all.clean.model.png')

    if interact:
        print('When you are done with and have closed the graphics window,')
        user_check = input('press enter to continue script\n')

    # First do gaintype=G to remove offsets between correlations
    os.system('rm -rf HD163296_cont.p1')
    gaincal(vis='HD163296_cont_all.ms',
            caltable='HD163296_cont.p1',
            solint='10000s',refant=antrefs,
            calmode='p', gaintype='G',
            combine='scan', minsnr=1.5,
            minblperant=3)

    if interact:
        plotms(vis='HD163296_cont.p1',xaxis='time', yaxis='phase', coloraxis='spw',
               gridrows=3, gridcols=3, iteraxis='antenna', plotrange=[0,0,-180,180.])

    if interact:
        print('When you are done with and have closed the graphics window,')
        user_check = input('press enter to continue script\n')

# Apply calibration and re-image
mystep = 5
if mystep in thesteps:
    casalog.post('Step '+str(mystep)+' '+step_title[mystep], 'INFO')
    print('Step ', mystep, step_title[mystep])

    applycal(vis='HD163296_cont_all.ms',
             gaintable='HD163296_cont.p1',
             calwt=False, applymode='calonly')

    os.system('rm -rf HD163296_cont.p1.clean*')
    tclean(vis='HD163296_cont_all.ms',
           imagename='HD163296_cont.p1.clean',
           specmode='mfs', deconvolver='multiscale',
           imsize=500, spw='0~3', scales=[0, 5, 10],
           cell=cellsize, weighting='briggs',
           robust=-0.5, interactive=interact,
           threshold='0.1mJy', niter=500,
           mask='HD163296_cont.clean.mask')

    peak=imstat(imagename='HD163296_cont.p1.clean.image',box='125,125,375,375')['max'][0]
    rms=imstat(imagename='HD163296_cont.p1.clean.image',box='10,10,125,125')['rms'][0]
    print('HD163296_cont.p1.clean.image peak = %7.4f mJy/beam, rms  = %7.4f mJy/beam, S/N = %7.4f' % (peak*1000, rms*1000, peak/rms))
    print('There should be an improvement')

# Phase self-cal with better model, shorter solint
mystep = 6
if mystep in thesteps:
    casalog.post('Step '+str(mystep)+' '+step_title[mystep], 'INFO')
    print('Step ', mystep, step_title[mystep])

    # Insert model from image in ms
    ft(vis='HD163296_cont_all.ms',
       model='HD163296_cont.p1.clean.model',
       usescratch=True)

    # check that model has saved after ft
    if interact:
        os.system('rm HD163296_cont.p1.clean.model.png')
        plotms(vis='HD163296_cont_all.ms', xaxis='uvwave', yaxis='amp', ydatacolumn='model',
               showgui=interact, plotfile='HD163296_cont.p1.clean.model.png')

    if interact:
        print('When you are done with and have closed the graphics window,')
        user_check = input('press enter to continue script\n')

    os.system('rm -rf HD163296_cont.p2')
    gaincal(vis='HD163296_cont_all.ms',
            caltable='HD163296_cont.p2',
            gaintable=['HD163296_cont.p1'],
            solint='1088s', refant=antrefs,
            calmode='p', gaintype='T',
            combine='scan', minsnr=1.5,
            minblperant=3)

    if interact:
        plotms(vis='HD163296_cont.p2',xaxis='time', yaxis='phase', coloraxis='spw',
           gridrows=3, gridcols=3, iteraxis='antenna', plotrange=[0,0,-180,180.])

    if interact:
        print('When you are done with and have closed the graphics window,')
        user_check = input('press enter to continue script\n')

# Apply phase calibration and re-image
mystep = 7
if mystep in thesteps:
    casalog.post('Step '+str(mystep)+' '+step_title[mystep], 'INFO')
    print('Step ', mystep, step_title[mystep])

    applycal(vis='HD163296_cont_all.ms',
             gaintable=['HD163296_cont.p1', 'HD163296_cont.p2'],
             calwt=False, applymode='calonly')

# Use spectral index imaging to prepare for amp cal
    os.system('rm -rf HD163296_cont.p2.clean*')
    tclean(vis='HD163296_cont_all.ms',
           imagename='HD163296_cont.p2.clean',
           specmode='mfs', deconvolver='multiscale',
           imsize=500, spw='0~3', scales=[0, 5, 10],
           cell=cellsize, weighting='briggs',
           robust=-0.5, interactive=interact,
           threshold='0.1mJy', niter=1000,
           mask='HD163296_cont.clean.mask')

    peak=imstat(imagename='HD163296_cont.p2.clean.image',box='125,125,375,375')['max'][0]
    rms=imstat(imagename='HD163296_cont.p2.clean.image',box='10,10,125,125')['rms'][0]
    print('HD163296_cont.p2.clean.image peak = %7.4f mJy/beam, rms  = %7.4f mJy/beam, S/N = %7.4f' % (peak*1000, rms*1000, peak/rms))
    print('More improvement?')

# Phase self-cal with better model, shorter solint
mystep = 8
if mystep in thesteps:
    casalog.post('Step '+str(mystep)+' '+step_title[mystep], 'INFO')
    print('Step ', mystep, step_title[mystep])

# Insert model from image in ms
    ft(vis='HD163296_cont_all.ms',
       model='HD163296_cont.p2.clean.model',
       usescratch=True)

    # check that model has saved after ft
    if interact:
        os.system('rm HD163296_cont.p2.clean.model.png')
        plotms(vis='HD163296_cont_all.ms', xaxis='uvwave', yaxis='amp', ydatacolumn='model',
               showgui=interact, plotfile='HD163296_cont.p2.clean.model.png')

    if interact:
        print('When you are done with and have closed the graphics window,')
        user_check = input('press enter to continue script\n')

    os.system('rm -rf HD163296_cont.p3')
    gaincal(vis='HD163296_cont_all.ms',
            caltable='HD163296_cont.p3',
            gaintable=['HD163296_cont.p1','HD163296_cont.p2'],
            solint='544s',refant=antrefs,
            calmode='p',gaintype='T',
            combine='scan', minsnr=1.5,
            minblperant=3)

    if interact:
        plotms(vis='HD163296_cont.p3',xaxis='time', yaxis='phase', coloraxis='spw',
               gridrows=3, gridcols=3, iteraxis='antenna', plotrange=[0,0,-180,180.])

    if interact:
        print('When you are done with and have closed the graphics window,')
        user_check = input('press enter to continue script\n')

# Apply best phase calibration and re-image
mystep = 9
if mystep in thesteps:
    casalog.post('Step '+str(mystep)+' '+step_title[mystep], 'INFO')
    print('Step ', mystep, step_title[mystep])

    applycal(vis='HD163296_cont_all.ms',
             gaintable=['HD163296_cont.p1','HD163296_cont.p2','HD163296_cont.p3'],
             calwt=False, applymode='calonly')

    os.system('rm -rf HD163296_cont.p3.clean*')
    tclean(vis='HD163296_cont_all.ms',
           imagename='HD163296_cont.p3.clean',
           specmode='mfs', deconvolver='multiscale',
           imsize=500, spw='0~3', scales=[0, 5, 10],
           cell=cellsize, weighting='briggs',
           robust=-0.5, interactive=interact,
           threshold='0.1mJy', niter=2000,
           mask='HD163296_cont.clean.mask')

    peak=imstat(imagename='HD163296_cont.p3.clean.image',box='125,125,375,375')['max'][0]
    rms=imstat(imagename='HD163296_cont.p3.clean.image',box='10,10,125,125')['rms'][0]
    print('HD163296_cont.p3.clean.image peak = %7.4f mJy/beam, rms  = %7.4f mJy/beam, S/N = %7.4f' % (peak*1000, rms*1000, peak/rms))
    print('More improvement?')

# Apply phase self-cal, perform amplitude self-cal
mystep = 10
if mystep in thesteps:
    casalog.post('Step '+str(mystep)+' '+step_title[mystep], 'INFO')
    print('Step ', mystep, step_title[mystep])

    solinta = '1088s'     # solution interval for amplitude selfcal, should be longer than phase

    # Insert model from image in ms
    ft(vis='HD163296_cont_all.ms',
       model='HD163296_cont.p3.clean.model',
       usescratch=True)

    # check that model has saved after ft
    if interact:
        os.system('rm HD163296_cont.p3.clean.model.png')
        plotms(vis='HD163296_cont_all.ms', xaxis='uvwave', yaxis='amp', ydatacolumn='model',
               showgui=interact, plotfile='HD163296_cont.p3.clean.model.png')

    if interact:
        print('When you are done with and have closed the graphics window,')
        user_check = input('press enter to continue script\n')

    os.system('rm -rf HD163296_cont.a1')
    gaincal(vis='HD163296_cont_all.ms',
            caltable='HD163296_cont.a1',
            gaintable=['HD163296_cont.p1','HD163296_cont.p2','HD163296_cont.p3'],
            solint=solinta, refant=antrefs,
            calmode='a',gaintype='T',
            combine='scan',
            minsnr=1.5, minblperant=4)

    if interact:
        plotms(vis='HD163296_cont.a1',xaxis='time', yaxis='amplitude',
            gridrows=3, gridcols=3, iteraxis='antenna', coloraxis='spw')

    if interact:
        print('When you are done with the graphics window,')
        user_check = input('press enter to continue script\n')

# Re-image after applying phase and amp calibration
mystep = 11
if mystep in thesteps:
    casalog.post('Step '+str(mystep)+' '+step_title[mystep], 'INFO')
    print('Step ', mystep, step_title[mystep])

    applycal(vis='HD163296_cont_all.ms',
             gaintable=['HD163296_cont.p1','HD163296_cont.p2','HD163296_cont.p3', 'HD163296_cont.a1'],
             calwt=False, applymode='calonly')

    # Image the corrected data
    os.system('rm -rf HD163296_cont.p3a1.clean*')
    tclean(vis='HD163296_cont_all.ms',
           imagename='HD163296_cont.p3a1.clean',
           specmode='mfs', deconvolver='multiscale',
           imsize=500, spw='0~3', scales=[0, 5, 10],
           cell=cellsize, weighting='briggs',
           robust=-0.5, interactive=interact,
           threshold='0.1mJy', niter=3000,
           mask='HD163296_cont.clean.mask')

    peak=imstat(imagename='HD163296_cont.p3a1.clean.image',box='125,125,375,375')['max'][0]
    rms=imstat(imagename='HD163296_cont.p3a1.clean.image',box='10,10,125,125')['rms'][0]
    print('HD163296_cont.p3a1.clean.image peak = %7.4f mJy/beam, rms  = %7.4f mJy/beam, S/N = %7.4f' % (peak*1000, rms*1000, peak/rms))
    print('More improvement?')

# Apply calibration to line data
mystep = 12
if mystep in thesteps:
    casalog.post('Step '+str(mystep)+' '+step_title[mystep], 'INFO')
    print('Step ', mystep, step_title[mystep])

    applycal(vis='HD163296_CO_new.ms',
             gaintable=['HD163296_cont.p1','HD163296_cont.p2','HD163296_cont.p3', 'HD163296_cont.a1'],
             calwt=False, applymode='calonly')

    if interact:
        print('Look at spw in frequency to identify line-free channels \nand enter these in continuum_fitspw')
        plotms(vis='HD163296_CO_new.ms',
               xaxis='channel', yaxis='amp',
               ydatacolumn='corrected',
               avgtime='99999', avgscan=True,
               coloraxis='corr')

        print('When you are done with the viewer,')
        print('close the window and')
        user_check = input('press enter to continue script\n')

# continuum setup
continum_fitspw = '0:**~**;**~**'

# Subtract continuum and re-plot
mystep = 13
if mystep in thesteps:
    casalog.post('Step '+str(mystep)+' '+step_title[mystep], 'INFO')
    print('Step ', mystep, step_title[mystep])

    if continum_fitspw != '0:**~**;**~**':
        
        os.system('rm -rf HD163296_CO_new.ms.contsub' )
        uvcontsub(vis='HD163296_CO_new.ms', spw='0', fitspw=continum_fitspw,
        excludechans=False, solint='int', fitorder=0, want_cont=False)
        
    else:   
        print('You did not set the continum_fitspw parameter.')
        print('Please set this parameters in the script before running step 13 again.')

# Rest frequency for CO(2-1)
co_21_restfreq = '230.538GHz'

# Threshold for line  cleaning (~3 x expected line rms)
linethresh = '***mJy'

# Velocity range over which to image the line
chanstart= '***km/s'
chanwidth= '***km/s'
nchan=  '***'

# Image CO cube
mystep = 14
if mystep in thesteps:
    casalog.post('Step '+str(mystep)+' '+step_title[mystep], 'INFO')
    print('Step ', mystep, step_title[mystep])

    if linethresh != '***mJy' and chanstart != '***km/s' and chanwidth != '***km/s' and nchan !=  '***':
        
        os.system('rm -rf ' + 'HD163296_CO_cube*')
        tclean(vis='HD163296_CO_new.ms.contsub' ,
               imagename='HD163296_CO_cube',
               specmode='cube', spw='0', 
               imsize=1000, cell='0.01arcsec',
               deconvolver='hogbom',
               start=chanstart, width=chanwidth, nchan=nchan,
               outframe='LSRK', veltype='radio', 
               restfreq=co_21_restfreq,
               weighting='briggs', robust=0.5, threshold=linethresh,
               niter=100000, cycleniter=500, 
               restoringbeam='common', pbcor=True,
               interactive=interact)

        # Determine synthesised beam for images using imhead
        # (if you didn't note it down from logger during clean)
        imhead('HD163296_CO_cube.image')
    else:
        print('You did not set the linethresh, chanstart, chanwidth and nchan parameters.')
        print('Please set these parameters in the script before running step 14 again.')

# Image CO cube with automasking
mystep = 15
if mystep in thesteps:
    casalog.post('Step '+str(mystep)+' '+step_title[mystep], 'INFO')
    print('Step ', mystep, step_title[mystep])

    print('Try automasking - check that the masks are reasonable, adjust if wanted.')

    if linethresh != '***mJy' and chanstart != '***km/s' and chanwidth != '***km/s' and nchan !=  '***':
        
        os.system('rm -rf ' + 'HD163296_CO_auto*')
        tclean(vis='HD163296_CO_new.ms.contsub' ,
               imagename='HD163296_CO_auto',
               specmode='cube', spw='0', imsize=1000, cell='0.01arcsec',
               deconvolver='hogbom',
               start=chanstart, width=chanwidth, nchan=nchan,
               outframe='LSRK', veltype='radio', restfreq=co_21_restfreq,
               niter=100000, cycleniter=500, 
               weighting='briggs', robust=0.5, threshold=linethresh,
               usemask='auto-multithresh',
               noisethreshold=3.3, sidelobethreshold=1.8,
               growiterations=400, lownoisethreshold=1.2,
               minbeamfrac = 0.5,
               restoringbeam='common', pbcor=True,
               interactive=interact)

        # Determine synthesised beam for images using imhead
        # (if you didn't note it down from logger during clean)
        imhead('HD163296_CO_auto.image')

        # Determine RMS noise in empty channel
        results = imstat('HD163296_CO_auto.image', chans='1')
        print (results)
        co_image_sigma = results['sigma'][0]
        print ("s.d. ", results['sigma'])
        print ("RMS ", results['rms'])

        # Open image in viewer to estimate spectral extent
        # of the emission for moment maps later
        if interact:
            print('Look at the cube to find which channels have line emission in')

            imview('HD163296_CO_auto.image')

            print('When you are done with and have closed the graphics window,')
            user_check = input('press enter to continue script\n')

    else:
        print('You did not set the linethresh, chanstart, chanwidth and nchan parameters.')
        print('Please set these parameters in the script before running step 15 again.')

# Set channels in which there is signal in the line datacube
moment_chans = '***~***'

mystep = 16
if mystep in thesteps:
    casalog.post('Step '+str(mystep)+' '+step_title[mystep], 'INFO')
    print('Step ', mystep, step_title[mystep])

    if interact:
        # Make spectrum using spectral profile tool in viewer,
        # Save region using View -> Regions -> File tab
        # (Regions panel may already be open)

        print('Create and save a region in the casa viewer in order to create a spectrum')

        imview('HD163296_CO_auto.image')

        print('When you are done with and have closed the graphics window,')
        user_check = input('press enter to continue script\n')

    # Also fit a gaussian to the line (interactively)

    # Fit gaussian to spectrum in region, e.g.
    spec_model = specfit(imagename='HD163296_CO_auto.image',
            region='spectrum_region.crtf', poly=-1,
            ngauss = 1, logresults=True)

# Make zeroth moment map
mystep = 17
if mystep in thesteps:
    casalog.post('Step '+str(mystep)+' '+step_title[mystep], 'INFO')
    print('Step ', mystep, step_title[mystep])

    # Open image in viewer to estimate spectral extent
    # of the emission

    if moment_chans != '***~***':
        
        os.system('rm -rf HD163296_CO_auto.image.mom0')
        immoments(imagename = 'HD163296_CO_auto.image',
                  outfile = 'HD163296_CO_auto.image.mom0',
                  moments =[0], chans= moment_chans)

        # View moment zero map in viewer
        if interact:
            imview( raster=[ {'file':'HD163296_CO_auto.image.mom0'}],
                    contour={'file':'HD163296_cont.p3a1.clean.image',
                             'base':0, 'unit':***,
                             'levels':[3,5,10,50]})

            print('When you are done with the viewer,')
            print('close the window and')
            user_check = input('press enter to continue script\n')


    else:
        print('You did not set the moment_chans parameter.')
        print('Please set this parameters in the script before running step 17 again.')


# Make first and second moment maps and export
mystep = 18
if mystep in thesteps:
    casalog.post('Step '+str(mystep)+' '+step_title[mystep], 'INFO')
    print('Step ', mystep, step_title[mystep])

    if moment_chans != '***~***':
        # Make first moment map for 12CO

        os.system('rm -rf HD163296_CO_auto.image.mom1')
        immoments(imagename='HD163296_CO_auto.image', moments=[1],
                  outfile='HD163296_CO_auto.image.mom1',
                  chans=moment_chans, includepix=[4.0 * co_image_sigma, ***])

        # Make second moment map for 12CO

        os.system('rm -rf HD163296_CO_auto.image.mom2')
        immoments(imagename='HD163296_CO_auto.image', moments=[2],
                  outfile='HD163296_CO_auto.image.mom2',
                  chans=moment_chans, includepix=[4.0 * co_image_sigma, ***])

        # View the moment maps using viewer

        if interact:
            imview( raster=[ {'file':'HD163296_CO_auto.image.mom0'},
                             {'file':'HD163296_CO_auto.image.mom1'},
                             {'file':'HD163296_CO_auto.image.mom2'} ],
                    contour={'file':'HD163296_cont.p3a1.clean.image',
                             'base':0, 'unit':***,
                             'levels':[3,5,10,50]} )

            print('When you are done with the viewer,')
            print('close the window and')
            user_check = input('press enter to continue script\n')

        # Export an image using exportfits (can add lines to export the moment maps too)

        os.system('rm -rf HD163296_CO_auto.image.fits')
        exportfits(imagename='HD163296_CO_auto.image',
                   fitsimage='HD163296_CO_auto.image.fits')


    else:
        print('You did not set the moment_chans parameter.')
        print('Please set this parameters in the script before running step 18 again.')

# Correct for the primary beam response
mystep = 19
if mystep in thesteps:
    casalog.post('Step '+str(mystep)+' '+step_title[mystep], 'INFO')
    print('Step ', mystep, step_title[mystep])

    os.system('rm -rf ' + 'HD163296_cont.p3a1.clean.image.pbcor')
    impbcor(imagename='HD163296_cont.p3a1.clean.image',
            pbimage='HD163296_cont.p3a1.clean.pb',
            mode='divide',
            outfile='HD163296_cont.p3a1.clean.image.pbcor')


# # Fit the continuum emission with a 2D gaussian
mystep = 20
if mystep in thesteps:
    casalog.post('Step '+str(mystep)+' '+step_title[mystep], 'INFO')
    print('Step ', mystep, step_title[mystep])

    imfit(imagename="HD163296_cont.p3a1.clean.image",
          region="imfit_region.crtf", logfile="contin_fit.log",
          model="HD163296_cont.p3a1.clean.image.imfit",
          residual="HD163296_cont.p3a1.clean.image.fitresid")

    if interact:
        print('Take a look at the image, the model fitted to it and the residuals.')
        print('Is the model a good fit?')

        imview( raster=[ {'file':'HD163296_cont.p3a1.clean.image'},
                         {'file':'HD163296_cont.p3a1.clean.image.imfit'},
                         {'file':'HD163296_cont.p3a1.clean.image.fitresid'} ])

        print('In this case it is a poor fit as we have tried to fit a Gaussian to a disc...')
        print('When you are done with the viewer,')
        print('close the window and')
        user_check = input('press enter to continue script\n')


# Make pv diagram using the viewer
mystep = 21
if mystep in thesteps:
    casalog.post('Step '+str(mystep)+' '+step_title[mystep], 'INFO')
    print('Step ', mystep, step_title[mystep])

    if interact:
        # make a pv diagram with the viewer
        # Click on the P/V tool button
        # Draw a slice across the source (blue to red shifted)
        # Go to menu => view => Regions => pV tab
        # Click “Generate P/V”
        # Change the averaging width and generate again
        # Save the image
        # (Note down the position angle of your slice!)
        print('Create a pv diagram in the viewer')

        imview('HD163296_CO_auto.image')

        print('When you are done with and have closed the graphics window,')
        user_check = input('press enter to continue script\n')


# these parameters control the pv diagram that you will make with impv.
# You can set these based on the parameters you find in the viewer in the previous step.
pv_centre = ['***', '***'] # centre of the pv slice in pixel co-ordinates.
pv_length = '***arcsec' # the length of the pv slice in arcesconds.
pv_pa = '***deg' # The position angle of the pv slice in degrees.

# Make pv diagram using task impv
mystep = 22
if mystep in thesteps:
    casalog.post('Step '+str(mystep)+' '+step_title[mystep], 'INFO')
    print('Step ', mystep, step_title[mystep])

    if pv_centre != ['***', '***'] and pv_length != '***arcsec' and pv_pa != '***deg':
        
        os.system('rm -rf HD163296_CO_auto.image.pv')
        impv(imagename='HD163296_CO_auto.image',
             outfile='HD163296_CO_auto.image.pv',
             mode='length', center=pv_centre,
             length=pv_length, pa=pv_pa)

    else:
        print('You did not set the pv_centre, pv_length and pv_pa parameters.')
        print('Please set these parameters in the script before running step 22 again.')

# Reproject an image to Galactic coordinates
mystep = 23
if mystep in thesteps:
    casalog.post('Step '+str(mystep)+' '+step_title[mystep], 'INFO')
    print('Step ', mystep, step_title[mystep])

    os.system('rm -rf HD163296_CO_auto.image.galactic')

    imregrid(imagename='HD163296_CO_auto.image',
             template='GALACTIC',
             output='HD163296_CO_auto.image.galactic')
